/************************************************************************************
 *  File:     main.c
 *  Purpose:  Cortex-M0/0+/1 main file.
 *            Replace with your code.
 *  Date:     05 July 2013
 *  Info:     If __NO_SYSTEM_INIT is defined in the Build options,
 *            the startup code will not branch to SystemInit()
 *            and the function can be removed
 ************************************************************************************/

typedef unsigned int uint32_t;

#define LED_Pin_Red 21
#define LED_Pin_Grn 22
#define LED_Pin_Blu 23

#define DELAYCNT 1000
#define SUBCOUNT 200

#define CLENR0 0x10001000

#define GPIO_BASE_ADR 0x50000000
#define GPIO_DIR      (GPIO_BASE_ADR+0x514)
#define GPIO_DIRSET   (GPIO_BASE_ADR+0x518)
#define GPIO_DIRCLR   (GPIO_BASE_ADR+0x51C)
#define GPIO_IN       (GPIO_BASE_ADR+0x510)
#define GPIO_OUT      (GPIO_BASE_ADR+0x504)
#define GPIO_OUTSET   (GPIO_BASE_ADR+0x508)
#define GPIO_OUTCLR   (GPIO_BASE_ADR+0x50C)
#define GPIO_PINCNF   (GPIO_BASE_ADR+0x700)

#ifndef __NO_SYSTEM_INIT
void SystemInit()
{}
#endif

static void SetUpLed(int iLedPin);
static void LedControl(int iLedPin, int bCondition);
static void Wait(int iCnt);

void main()
{

  // Tell chip that we are running as Mode 0.
  uint32_t *pC;
  pC = (uint32_t *) (CLENR0);
  *pC = 0x3FF00;

  int cnt;
  cnt = 0;

  uint32_t *pV;
  pV = (uint32_t *)(GPIO_PINCNF);
  uint32_t v = *pV;

  SetUpLed(LED_Pin_Red);
  SetUpLed(LED_Pin_Grn);
  SetUpLed(LED_Pin_Blu);

  while(1)
  {
    LedControl(LED_Pin_Red, 1);
    Wait(DELAYCNT);
    LedControl(LED_Pin_Red, 0);
    LedControl(LED_Pin_Grn, 1);
    Wait(DELAYCNT);
    LedControl(LED_Pin_Grn, 0);
    LedControl(LED_Pin_Blu, 1);
    Wait(DELAYCNT);
    LedControl(LED_Pin_Blu, 0);
    cnt++;
  }
}

static void SetUpLed(int iLedPin)
{
    uint32_t *pCnf;

    pCnf = (uint32_t *) (GPIO_PINCNF + (4 * iLedPin));
    *pCnf = 0x00000001;  // Make the pin an output, everything else default.
    LedControl(iLedPin, 0);
}

static void LedControl(int iLedPin, int bCondition)
{
    uint32_t *pClr;
    uint32_t *pSet;
    pClr = (uint32_t *) (GPIO_OUTCLR);
    pSet = (uint32_t *) (GPIO_OUTSET);
    uint32_t v = 1 << iLedPin;
    if(bCondition) *pSet = v;
    else           *pClr = v;
}

static void Wait(int iCnt)
{
    int i, j;
    for(i = 0; i < iCnt; i++)
    {
        for(j = 0; j < SUBCOUNT; j++);
    }
}
